package com.ibm.websphere.query.base;

import java.util.ArrayList;
import java.util.List;

/**
 * <P>The Predicate class represents a predicate in the query framework. It is derived from 
 * PredicateBase.
 *	
 * <P>This class contains an operator and predicate bases to describe a typical predicate in a 
 * query. The operator is usually either "AND" or "OR". For example, a predicate "(age > 30) AND
 * (last name = Smith)" will be decomposed	as operator = Operator("AND"), and predicate bases = 
 * Condition(Attribute("age"), Operator(">"), Value("30", Constants.DATATYPE_INTEGER)), 
 * Condition(Attribute("last name"), Operator("="), Value("Smith", Constants.DATATYPE_CHAR))
 */
public class Predicate extends PredicateBase implements IPredicate
{
  private IOperator operator = null;
  private List predicateBases = new ArrayList();

  /**
   * Creates a new Predicate instance, default constructor.
   */
  public Predicate()
  {
  }

  /**
   * Creates a new Predicate instance with the given operator and array of predicate bases.
   */
  public Predicate(IOperator operator, PredicateBase[] predicateBases)
  {
    this.operator = operator;
    setPredicateBases(predicateBases);
  }

  /**
   * Creates a new Predicate instance with the given operator and list of predicate bases.
   */
  public Predicate(IOperator operator, List predicateBases)
  {
    this.operator = operator;
    setPredicateBases(predicateBases);
  }

  /**
   * Adds a predicate base to the predicate base list.
   */
  public void addPredicateBase(PredicateBase p)
  {
    predicateBases.add(p);
  }
  /**
   * Adds an array of predicate bases to the predicate base list.
   */
  public void addPredicateBases(List predicateBases)
  {
    this.predicateBases.addAll(predicateBases);
  }
  /**
   * Builds a query string for the predicate.
   */
  public String buildString(ISelectQueryCallback builder) throws QueryException
  {
    return builder.buildPredicate(this);
  }
  public void buildString(ISelectQueryCallback builder, StringBuffer sb) throws QueryException
  {
    builder.buildPredicate(this, sb);
  }
  /**
   * Tests for equality with the given object.
   */
  public boolean equals(Object o)
  {
    boolean eq = super.equals(o);

    if ((eq == true) && (o instanceof Predicate))
    {
      Predicate p = (Predicate) o;

      if (operator != null)
      {
        eq = operator.equals(p.operator);
      }
      else
      {
        eq = (p.operator == null);
      }

      if (eq)
      {
        eq = predicateBases.equals(p.predicateBases);
      }
    }
    else
    {
      eq = false;
    }

    return eq;
  }
  /**
   * Returns the operator.
   */
  public IOperator getOperator()
  {
    return operator;
  }
  /**
   * Returns the array of predicate bases.
   */
  public PredicateBase[] getPredicateBases()
  {
    return (PredicateBase[]) predicateBases.toArray(new PredicateBase[predicateBases.size()]);
  }
  /**
   * Computes a hash code for the predicate.
   */
  public int hashCode()
  {
    int hashCode = super.hashCode();

    if (operator != null)
      hashCode = hashCode ^ operator.hashCode();

    for (int i = 0; i < predicateBases.size(); i++)
    {
      hashCode = hashCode ^ predicateBases.get(i).hashCode();
    }

    return hashCode;
  }
  /**
   * Sets the operator of the predicate.
   */
  public void setOperator(IOperator newOperator)
  {
    operator = newOperator;
  }
  /**
   * Sets the predicate bases of the predicate.
   */
  public void setPredicateBases(List predicateBases)
  {
    this.predicateBases = predicateBases;
  }

  /**
   * Sets the predicate bases of the predicate.
   */
  public void setPredicateBases(PredicateBase[] predicateBases)
  {
    this.predicateBases.clear();
    for (int i = 0; i < predicateBases.length; i++)
    {
      this.predicateBases.add(predicateBases[i]);
    }
  }
}
