/*********************************************************************
 * Licensed Materials - Property of IBM                              *
 *                                                                   *
 * 5724-B88                                                          *
 *                                                                   *
 *  Copyright IBM Corp.  2001, 2002    All Rights Reserved.         *
 *                                                                   *
 * US Government Users Restricted Rights - Use, duplication or       *
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp. *
 *********************************************************************/

package com.ibm.websphere.query.serialization;

import com.ibm.websphere.query.base.IArithmeticCondition;
import com.ibm.websphere.query.base.IAttribute;
import com.ibm.websphere.query.base.ICondition;
import com.ibm.websphere.query.base.IFunction;
import com.ibm.websphere.query.base.IFunctionElement;
import com.ibm.websphere.query.base.ILeftOperand;
import com.ibm.websphere.query.base.ILimit;
import com.ibm.websphere.query.base.IOperator;
import com.ibm.websphere.query.base.IOrderBy;
import com.ibm.websphere.query.base.IOrderByElement;
import com.ibm.websphere.query.base.IOrderByExpression;
import com.ibm.websphere.query.base.IPredicate;
import com.ibm.websphere.query.base.IQuery;
import com.ibm.websphere.query.base.IRightOperand;
import com.ibm.websphere.query.base.ISelectQuery;
import com.ibm.websphere.query.base.ISelectQueryCallback;
import com.ibm.websphere.query.base.IValue;
import com.ibm.websphere.query.base.PredicateBase;
import com.ibm.websphere.query.base.QueryException;

import com.ibm.websphere.query.callbacks.SelectQueryCallbackAdapter;

/**
 * This is an XML builder that will generate an XML representation of a
 * SelectQuery.  It will also append the name/value pairs in the criteria
 * map for a UserSelectQuery.
 */
public class XMLSelectQueryCallback extends SelectQueryCallbackAdapter implements ISelectQueryCallback
{
  /**
   * Whether or not to add whitespace and newlines between elements.
   */
  private boolean addWhiteSpace;

  /**
   * The line separator
   */
  private String lineSeparator = "\n";

  /**
   * The indention string
   */
  private String indentionStr = "  ";

  /**
   * The current indention level
   */
  private StringBuffer indention = new StringBuffer();

  /**
   * Default constructor
   */
  public XMLSelectQueryCallback()
  {
    super();
  }

  /**
   * @param addWhiteSpace Whether or not to add whitespace and line separators between elements.
   */
  public XMLSelectQueryCallback(boolean addWhiteSpace)
  {
    super();
    this.addWhiteSpace = addWhiteSpace;
  }

  /* (non-Javadoc)
   * @see com.ibm.websphere.query.base.ISelectQueryCallback#buildAttribute(com.ibm.websphere.query.base.IAttribute)
   */
  public String buildAttribute(IAttribute attribute) throws QueryException
  {
    if (attribute == null)
      return "";

    StringBuffer sb = new StringBuffer();

    // Add whitespace
    appendIndention(sb);

    sb.append("<");
    sb.append(XMLConstants.ATTRIBUTE_ELEMENT);

    // Append the name attribute if it exists
    String attrValue = attribute.getName();
    if (attrValue != null && attrValue.trim().length() > 0)
    {
      sb.append(" ");
      sb.append(XMLConstants.NAME_ATTR);
      sb.append("=\"");
      sb.append(escapeStringForXML(attrValue));
      sb.append("\"");
    }

    // Append the data type attribute
    sb.append(" ");
    sb.append(XMLConstants.DATA_TYPE_ATTR);
    sb.append("=\"");
    sb.append(attribute.getDataType());
    sb.append("\"");
    sb.append("/>");

    // Add whitespace
    newLine(sb);

    return sb.toString();
  }

  /* (non-Javadoc)
   * @see com.ibm.websphere.query.base.ISelectQueryCallback#buildCondition(com.ibm.websphere.query.base.ICondition)
   */
  public String buildCondition(ICondition Condition) throws QueryException
  {
    if (Condition == null)
      return "";

    StringBuffer sb = new StringBuffer();

    // Add whitespace
    appendIndention(sb);

    sb.append("<");
    sb.append(XMLConstants.CONDITION_ELEMENT);
    sb.append(">");

    // Add whitespace
    newLineIncreaseIndention(sb);

    // Build the left operand
    sb.append(buildLeftOperand(Condition.getLeftOperand()));

    // Build the operator
    sb.append(buildOperator(Condition.getOperator()));

    // Build the right operands
    IRightOperand[] rightOperands = Condition.getRightOperands();
    for (int i = 0; i < rightOperands.length; i++)
    {
      sb.append(buildRightOperand(rightOperands[i]));
    }

    // Add whitespace
    decreaseAndAppendIndention(sb);

    // Close the element
    sb.append("</");
    sb.append(XMLConstants.CONDITION_ELEMENT);
    sb.append(">");

    // Add whitespace
    newLine(sb);

    return sb.toString();
  }

  private String buildLeftOperand(ILeftOperand leftOperand) throws QueryException
  {
    if (leftOperand == null)
      return "";

    StringBuffer sb = new StringBuffer();

    if (leftOperand instanceof IArithmeticCondition)
    {
      sb.append(buildArithmeticCondition((IArithmeticCondition) leftOperand));
    }
    else if (leftOperand instanceof IAttribute)
    {
      sb.append(buildAttribute((IAttribute) leftOperand));
    }
    else if (leftOperand instanceof IFunction)
    {
      sb.append(buildFunction((IFunction) leftOperand));
    }
    else if (leftOperand instanceof IValue)
    {
      sb.append(buildValue((IValue) leftOperand));
    }

    return sb.toString();
  }

  private String buildRightOperand(IRightOperand rightOperand) throws QueryException
  {
    if (rightOperand == null)
      return "";

    StringBuffer sb = new StringBuffer();

    if (rightOperand instanceof IArithmeticCondition)
    {
      sb.append(buildArithmeticCondition((IArithmeticCondition) rightOperand));
    }
    else if (rightOperand instanceof IAttribute)
    {
      sb.append(buildAttribute((IAttribute) rightOperand));
    }
    else if (rightOperand instanceof IFunction)
    {
      sb.append(buildFunction((IFunction) rightOperand));
    }
    else if (rightOperand instanceof IValue)
    {
      sb.append(buildValue((IValue) rightOperand));
    }

    return sb.toString();
  }

  private String buildFunctionElement(IFunctionElement functionElement) throws QueryException
  {
    if (functionElement == null)
      return "";

    StringBuffer sb = new StringBuffer();

    if (functionElement instanceof IArithmeticCondition)
    {
      sb.append(buildArithmeticCondition((IArithmeticCondition) functionElement));
    }
    else if (functionElement instanceof IAttribute)
    {
      sb.append(buildAttribute((IAttribute) functionElement));
    }
    else if (functionElement instanceof IFunction)
    {
      sb.append(buildFunction((IFunction) functionElement));
    }
    else if (functionElement instanceof IValue)
    {
      sb.append(buildValue((IValue) functionElement));
    }

    return sb.toString();
  }

  private String buildOrderByElement(IOrderByElement orderByElement) throws QueryException
  {
    if (orderByElement == null)
      return "";

    StringBuffer sb = new StringBuffer();

    if (orderByElement instanceof IAttribute)
    {
      sb.append(buildAttribute((IAttribute) orderByElement));
    }
    else if (orderByElement instanceof IFunction)
    {
      sb.append(buildFunction((IFunction) orderByElement));
    }

    return sb.toString();
  }

  /* (non-Javadoc)
   * @see com.ibm.websphere.query.base.ISelectQueryCallback#buildFunction(com.ibm.websphere.query.base.IFunction)
   */
  public String buildFunction(IFunction function) throws QueryException
  {
    if (function == null)
      return "";

    StringBuffer sb = new StringBuffer();

    // Add whitespace
    appendIndention(sb);

    sb.append("<");
    sb.append(XMLConstants.FUNCTION_ELEMENT);

    // Append the name attribute if it exists
    String attrValue = function.getName();
    if (attrValue != null && attrValue.trim().length() > 0)
    {
      sb.append(" ");
      sb.append(XMLConstants.NAME_ATTR);
      sb.append("=\"");
      sb.append(escapeStringForXML(attrValue));
      sb.append("\"");
    }

    // Append the data type attribute
    sb.append(" ");
    sb.append(XMLConstants.DATA_TYPE_ATTR);
    sb.append("=\"");
    sb.append(function.getDataType());
    sb.append("\">");

    // Add whitespace
    newLineIncreaseIndention(sb);

    // Build the function elements
    IFunctionElement[] functionElements = function.getFunctionElements();
    for (int i = 0; i < functionElements.length; i++)
    {
      sb.append(buildFunctionElement(functionElements[i]));
    }

    // Add whitespace
    decreaseAndAppendIndention(sb);

    // Close the element
    sb.append("</");
    sb.append(XMLConstants.FUNCTION_ELEMENT);
    sb.append(">");

    // Add whitespace
    newLine(sb);

    return sb.toString();
  }

  private void newLineIncreaseIndention(StringBuffer sb)
  {
    // Add a new line and increase the indention
    if (shouldAddWhiteSpace())
    {
      sb.append(lineSeparator);
      increaseIndention();
    }
  }

  private void newLine(StringBuffer sb)
  {
    // Add a new line
    if (shouldAddWhiteSpace())
    {
      sb.append(lineSeparator);
    }
  }

  private void appendIndention(StringBuffer sb)
  {
    if (shouldAddWhiteSpace())
    {
      sb.append(indention);
    }
  }

  private void decreaseAndAppendIndention(StringBuffer sb)
  {
    // Decrease the indention and append it
    if (shouldAddWhiteSpace())
    {
      decreaseIndention();
      sb.append(indention);
    }
  }

  /* (non-Javadoc)
   * @see com.ibm.websphere.query.base.ISelectQueryCallback#buildArithmeticCondition(com.ibm.websphere.query.base.IArithmeticCondition)
   */
  public String buildArithmeticCondition(IArithmeticCondition arithmeticCondition) throws QueryException
  {
    if (arithmeticCondition == null)
      return "";

    StringBuffer sb = new StringBuffer();

    // Add whitespace
    appendIndention(sb);

    sb.append("<");
    sb.append(XMLConstants.ARITHMETIC_CONDITION_ELEMENT);

    // Append the operation attribute if it exists
    String attrValue = arithmeticCondition.getArithmeticOperation();
    if (attrValue != null && attrValue.trim().length() > 0)
    {
      sb.append(" ");
      sb.append(XMLConstants.OPERATION_ATTR);
      sb.append("=\"");
      sb.append(escapeStringForXML(attrValue));
      sb.append("\"");
    }

    // Append the data type attribute
    sb.append(" ");
    sb.append(XMLConstants.DATA_TYPE_ATTR);
    sb.append("=\"");
    sb.append(arithmeticCondition.getDataType());
    sb.append("\">");

    // Add whitespace
    newLineIncreaseIndention(sb);

    // Build the left operands
    ILeftOperand[] leftOperands = arithmeticCondition.getArithmeticConditionElements();
    for (int i = 0; i < leftOperands.length; i++)
    {
      sb.append(buildLeftOperand(leftOperands[i]));
    }

    // Add whitespace
    decreaseAndAppendIndention(sb);

    // Close the element
    sb.append("</");
    sb.append(XMLConstants.ARITHMETIC_CONDITION_ELEMENT);
    sb.append(">");

    // Add whitespace
    newLine(sb);

    return sb.toString();
  }

  /* (non-Javadoc)
   * @see com.ibm.websphere.query.base.ISelectQueryCallback#buildOperator(com.ibm.websphere.query.base.IOperator)
   */
  public String buildOperator(IOperator operator) throws QueryException
  {
    if (operator == null)
      return "";

    StringBuffer sb = new StringBuffer();

    // Add whitespace
    appendIndention(sb);

    sb.append("<");
    sb.append(XMLConstants.OPERATOR_ELEMENT);

    // Append the name attribute if it exists
    String attrValue = operator.getName();
    if (attrValue != null && attrValue.trim().length() > 0)
    {
      sb.append(" ");
      sb.append(XMLConstants.NAME_ATTR);
      sb.append("=\"");
      sb.append(escapeStringForXML(attrValue));
      sb.append("\"");
    }
    sb.append("/>");

    // Add whitespace
    newLine(sb);

    return sb.toString();
  }

  /* (non-Javadoc)
   * @see com.ibm.websphere.query.base.ISelectQueryCallback#buildOrderBy(com.ibm.websphere.query.base.IOrderBy)
   */
  public String buildOrderBy(IOrderBy orderBy) throws QueryException
  {
    if (orderBy == null)
      return "";

    StringBuffer sb = new StringBuffer();

    // Add whitespace
    appendIndention(sb);

    sb.append("<");
    sb.append(XMLConstants.ORDER_BY_ELEMENT);
    sb.append(">");

    // Add whitespace
    newLineIncreaseIndention(sb);

    // Build the order by expressions
    IOrderByExpression[] orderByExpressions = orderBy.getOrderByExpressions();
    for (int i = 0; i < orderByExpressions.length; i++)
    {
      sb.append(buildOrderByExpression(orderByExpressions[i]));
    }

    // Add whitespace
    decreaseAndAppendIndention(sb);

    // Close the element
    sb.append("</");
    sb.append(XMLConstants.ORDER_BY_ELEMENT);
    sb.append(">");

    // Add whitespace
    newLine(sb);

    return sb.toString();
  }

  /* (non-Javadoc)
   * @see com.ibm.websphere.query.base.ISelectQueryCallback#buildOrderBy(com.ibm.websphere.query.base.ISelectQuery)
   */
  public String buildOrderBy(ISelectQuery selectQuery) throws QueryException
  {
    if (selectQuery == null)
      return "";

    IOrderBy orderBy = selectQuery.getOrderBy();
    if (orderBy != null)
    {
      return buildOrderBy(orderBy);
    }
    return "";
  }

  /* (non-Javadoc)
   * @see com.ibm.websphere.query.base.ISelectQueryCallback#buildOrderByExpression(com.ibm.websphere.query.base.IOrderByExpression)
   */
  public String buildOrderByExpression(IOrderByExpression orderByExpression) throws QueryException
  {
    if (orderByExpression == null)
      return "";

    if (orderByExpression == null)
      return "";

    StringBuffer sb = new StringBuffer();

    // Add whitespace
    appendIndention(sb);

    sb.append("<");
    sb.append(XMLConstants.ORDER_BY_EXPRESSION_ELEMENT);
    sb.append(">");

    // Add whitespace
    newLineIncreaseIndention(sb);

    // Build the operator
    sb.append(buildOperator(orderByExpression.getOperator()));

    // Build the order by element
    sb.append(buildOrderByElement(orderByExpression.getOrderByElement()));

    // Add whitespace
    decreaseAndAppendIndention(sb);

    // Close the element
    sb.append("</");
    sb.append(XMLConstants.ORDER_BY_EXPRESSION_ELEMENT);
    sb.append(">");

    // Add whitespace
    newLine(sb);

    return sb.toString();
  }

  /* (non-Javadoc)
   * @see com.ibm.websphere.query.base.ISelectQueryCallback#buildLimit(com.ibm.websphere.query.base.ILimit)
   */
  public String buildLimit(ILimit limit) throws QueryException
  {
    if (limit == null)
      return "";

    StringBuffer sb = new StringBuffer();

    // Add whitespace
    appendIndention(sb);

    sb.append("<");
    sb.append(XMLConstants.LIMIT_ELEMENT);

    // Append the value attribute
    sb.append(" ");
    sb.append(XMLConstants.VALUE_ATTR);
    sb.append("=\"");
    sb.append(limit.getLimitValue());
    sb.append("\"");
    sb.append("/>");

    // Add whitespace
    newLine(sb);

    return sb.toString();
  }

  /* (non-Javadoc)
   * @see com.ibm.websphere.query.base.ISelectQueryCallback#buildLimit(com.ibm.websphere.query.base.ISelectQuery)
   */
  public String buildLimit(ISelectQuery selectQuery) throws QueryException
  {
    if (selectQuery == null)
      return "";

    ILimit limit = selectQuery.getLimit();
    if (limit != null)
    {
      return buildLimit(limit);
    }
    return "";
  }

  /* (non-Javadoc)
   * @see com.ibm.websphere.query.base.ISelectQueryCallback#buildPredicate(com.ibm.websphere.query.base.IPredicate)
   */
  public String buildPredicate(IPredicate predicate) throws QueryException
  {
    if (predicate == null)
      return "";

    StringBuffer sb = new StringBuffer();

    // Add whitespace
    appendIndention(sb);

    sb.append("<");
    sb.append(XMLConstants.PREDICATE_ELEMENT);
    sb.append(">");

    // Add whitespace
    newLineIncreaseIndention(sb);

    // Build the operator
    sb.append(buildOperator(predicate.getOperator()));

    // Build the predicate bases
    PredicateBase[] predicateBases = predicate.getPredicateBases();
    for (int i = 0; i < predicateBases.length; i++)
    {
      if (predicateBases[i] instanceof ICondition)
      {
        sb.append(buildCondition((ICondition) predicateBases[i]));
      }
      else if (predicateBases[i] instanceof IPredicate)
      {
        sb.append(buildPredicate((IPredicate) predicateBases[i]));
      }
    }

    // Add whitespace
    decreaseAndAppendIndention(sb);

    // Close the element
    sb.append("</");
    sb.append(XMLConstants.PREDICATE_ELEMENT);
    sb.append(">");

    // Add whitespace
    newLine(sb);

    return sb.toString();
  }

  /* (non-Javadoc)
   * @see com.ibm.websphere.query.base.ISelectQueryCallback#buildPredicate(com.ibm.websphere.query.base.IQuery)
   */
  public String buildPredicate(IQuery query) throws QueryException
  {
    if (query == null)
      return "";

    PredicateBase predicateBase = query.getPredicate();
    if (predicateBase != null)
    {
      if (predicateBase instanceof ICondition)
      {
        return buildCondition((ICondition) predicateBase);
      }
      else if (predicateBase instanceof IPredicate)
      {
        return buildPredicate((IPredicate) predicateBase);
      }
    }
    return "";
  }

  /* (non-Javadoc)
   * @see com.ibm.websphere.query.base.ISelectQueryCallback#buildQuery(com.ibm.websphere.query.base.IQuery)
   */
  public String buildQuery(IQuery query) throws QueryException
  {
    if (query == null)
      return "";

    StringBuffer sb = new StringBuffer();

    // Append encoding
    sb.append(XMLConstants.XML_VERSION);
    newLine(sb);

    sb.append("<");
    sb.append(XMLConstants.QUERY_ELEMENT);

    // Append the query class
    String queryClass = query.getClass().getName();
    sb.append(" ");
    sb.append(XMLConstants.CLASS_ATTR);
    sb.append("=\"");
    sb.append(queryClass);
    sb.append("\">");

    // Add whitespace
    newLineIncreaseIndention(sb);

    // Build the predicate base
    PredicateBase predicateBase = query.getPredicate();
    if (predicateBase != null)
    {
      if (predicateBase instanceof ICondition)
      {
        sb.append(buildCondition((ICondition) predicateBase));
      }
      else if (predicateBase instanceof IPredicate)
      {
        sb.append(buildPredicate((IPredicate) predicateBase));
      }
    }

    // Add whitespace
    decreaseAndAppendIndention(sb);

    // Close the element
    sb.append("</");
    sb.append(XMLConstants.QUERY_ELEMENT);
    sb.append(">");

    return sb.toString();
  }

  /* (non-Javadoc)
   * @see com.ibm.websphere.query.base.ISelectQueryCallback#buildSelectQuery(com.ibm.websphere.query.base.ISelectQuery)
   */
  public String buildSelectQuery(ISelectQuery selectQuery) throws QueryException
  {
    if (selectQuery == null)
      return "";

    StringBuffer sb = new StringBuffer();

    // Append encoding
    sb.append(XMLConstants.XML_VERSION);
    newLine(sb);

    sb.append("<");
    sb.append(XMLConstants.QUERY_ELEMENT);

    // Append the query class
    String queryClass = selectQuery.getClass().getName();
    sb.append(" ");
    sb.append(XMLConstants.CLASS_ATTR);
    sb.append("=\"");
    sb.append(queryClass);
    sb.append("\"");
    sb.append(">");

    // Add whitespace
    newLineIncreaseIndention(sb);

    // Build the predicate base
    PredicateBase predicateBase = selectQuery.getPredicate();
    if (predicateBase != null)
    {
      if (predicateBase instanceof ICondition)
      {
        sb.append(buildCondition((ICondition) predicateBase));
      }
      else if (predicateBase instanceof IPredicate)
      {
        sb.append(buildPredicate((IPredicate) predicateBase));
      }
    }

    // Build the order by
    sb.append(buildOrderBy(selectQuery.getOrderBy()));

    // Build the limit
    sb.append(buildLimit(selectQuery.getLimit()));

    // Add whitespace
    decreaseAndAppendIndention(sb);

    // Close the element
    sb.append("</");
    sb.append(XMLConstants.QUERY_ELEMENT);
    sb.append(">");

    return sb.toString();
  }

  /* (non-Javadoc)
   * @see com.ibm.websphere.query.base.ISelectQueryCallback#buildValue(com.ibm.websphere.query.base.IValue)
   */
  public String buildValue(IValue value) throws QueryException
  {
    if (value == null)
      return "";

    StringBuffer sb = new StringBuffer();

    // Add whitespace
    appendIndention(sb);

    sb.append("<");
    sb.append(XMLConstants.VALUE_ELEMENT);

    // Append the property name attribute if it exists
    String attrValue = value.getPropertyName();
    if (attrValue != null && attrValue.trim().length() > 0)
    {
      sb.append(" ");
      sb.append(XMLConstants.PROPERTY_NAME_ATTR);
      sb.append("=\"");
      sb.append(escapeStringForXML(attrValue));
      sb.append("\"");
    }

    // Append the data type attribute
    sb.append(" ");
    sb.append(XMLConstants.DATA_TYPE_ATTR);
    sb.append("=\"");
    sb.append(value.getDataType());
    sb.append("\"");

    // Append the value string attribute if it exists
    attrValue = value.getValueString();
    if (attrValue != null && attrValue.trim().length() > 0)
    {
      sb.append(" ");
      sb.append(XMLConstants.VALUE_ATTR);
      sb.append("=\"");
      sb.append(escapeStringForXML(attrValue));
      sb.append("\"");
    }

    sb.append("/>");

    // Add whitespace
    newLine(sb);

    return sb.toString();
  }

  /**
   * @return Whether or not to add whitespace between the elements.
   */
  public boolean shouldAddWhiteSpace()
  {
    return addWhiteSpace;
  }

  /**
   * @param Whether or not to add whitespace between the elements.
   */
  public void setAddWhiteSpace(boolean b)
  {
    addWhiteSpace = b;
  }

  private void increaseIndention()
  {
    // Increase the indention
    indention.append(indentionStr);
  }

  private void decreaseIndention()
  {
    // Decrease the indention length
    indention.setLength(indention.length() - indentionStr.length());
  }

  /**
   * @return
   */
  public String getIndentionStr()
  {
    return indentionStr;
  }

  /**
   * @return
   */
  public String getLineSeparator()
  {
    return lineSeparator;
  }

  /**
   * @param string
   */
  public void setIndentionStr(String string)
  {
    indentionStr = string;
  }

  /**
   * @param string
   */
  public void setLineSeparator(String string)
  {
    lineSeparator = string;
  }

  /**
   * Special characters not allowed in the XML stream.
   * We should handle DBCS characters here too.
   *
   * @param input String to convert
   * @return converted
   */
  public static String escapeStringForXML( String input )
  {
	StringBuffer converted = new StringBuffer();

	for( int i=0; i<input.length(); i++ )
	{
	  char ch = input.charAt(i);
	  switch( ch )
	  {
		case '&':
		  converted.append( "&amp;" );
		  break;
		case '<':
		  converted.append( "&lt;" );
		  break;
		case '>':
		  converted.append( "&gt;" );
		  break;
		case '"':
		  converted.append( "&quot;" );
		  break;
		case '\n':
		  converted.append( ch );
		  break;
		default:
		  if( !Character.isLetterOrDigit(ch) )
			converted.append( "&#" ).append( (int)ch ).append( ";" );
		  else
			converted.append( ch );
	  }
	}

	return converted.toString();
  }

}