/*
 * SimpleValue.java
 *
 * This file is part of the specification for the
 * Java Content Repository API
 */
package javax.jcr;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;

/**
 * This class is the superclass of the type-specific
 * classes implementing the <code>Value</code> interfaces.
 *
 * @see javax.jcr.Value
 * @see javax.jcr.StringValue
 * @see javax.jcr.LongValue
 * @see javax.jcr.DoubleValue
 * @see javax.jcr.BooleanValue
 * @see javax.jcr.DateValue
 * @see javax.jcr.BinaryValue
 * @see javax.jcr.SoftLinkValue
 *
 * @author Stefan Guggisberg
 */
public abstract class BaseValue implements Value {

    protected static final String DEFAULT_ENCODING = "UTF-8";

    private static final short STATE_UNDEFINED = 0;
    private static final short STATE_VALUE_CONSUMED = 1;
    private static final short STATE_STREAM_CONSUMED = 2;

    private short state = STATE_UNDEFINED;

    protected final int type;

    protected InputStream stream = null;

    /**
     * Package-private default constructor.
     *
     * @param type The type of this value.
     */
    BaseValue(int type) {
        this.type = type;
    }

    /**
     * Checks if the non-stream value of this instance has already been
     * consumed (i.e. if any getter methods except <code>{@link #getStream()}/<code> and
     * <code>{@link #getType()}</code> have been previously called at least once) and
     * sets the state to <code>STATE_STREAM_CONSUMED</code>.
     * @throws IllegalStateException if any getter methods other than
     *         <code>getStream()</code> and <code>getType()</code> have been
     *         previously called at least once.
     */
    protected void setStreamConsumed() throws IllegalStateException {
        if (state == STATE_VALUE_CONSUMED) {
            throw new IllegalStateException("non-stream value has already been consumed");
        }
        state = STATE_STREAM_CONSUMED;
    }

    /**
     * Checks if the stream value of this instance has already been
     * consumed (i.e. if <code>{@link #getStream()}/<code> has been previously called
     * at least once) and sets the state to <code>STATE_VALUE_CONSUMED</code>.
     * @throws IllegalStateException if <code>getStream()</code> has been
     *         previously called at least once.
     */
    protected void setValueConsumed() throws IllegalStateException {
        if (state == STATE_STREAM_CONSUMED) {
            throw new IllegalStateException("stream value has already been consumed");
        }
        state = STATE_VALUE_CONSUMED;
    }

    //----------------------------------------------------------------< Value >
    /**
     * @see Value#getType
     */
    public int getType() {
        return type;
    }

    /**
     * @see Value#getStream
     */
    public InputStream getStream() throws ValueFormatException, RepositoryException {             
        setStreamConsumed();     
                      
        if (stream != null) {
            return stream;
        }
        
        state = STATE_UNDEFINED;
        String s = getString();
        state = STATE_UNDEFINED;
        setStreamConsumed();

        try {
            // convert via string            
            stream = new ByteArrayInputStream(s.getBytes(DEFAULT_ENCODING));            
            return stream;
        } catch (UnsupportedEncodingException e) {
            throw new RepositoryException(DEFAULT_ENCODING + " not supported on this platform", e);
        }
    }
}
