/*
 * Credentials.java
 *
 * This file is part of the specification for the
 * Java Content Repository API
 */
package javax.jcr;

import java.io.Serializable;

/**
 * Base interface for all credentials that may be passed to the
 * {@link Repository#login(Credentials)} method.
 * Represents the credentials that are passed on login to the Repository.
 * Provides a minimal standard method for authenticating against a
 * repository (i.e., using a user ID and password). Additional attributes may
 * be used by the repository to, for example, set a token that can then be
 * passed back and forth once authentication has been completed (thus enabling
 * later authorization without re-authentication). In addition any class that
 * implements this interface can also, of course, add its own methods to deal
 * with more advanced authentication/authorization mechanisms.
 *
 * @author Peeter Piegaze
 * @author Stefan Guggisberg
 */
public interface Credentials extends Serializable {

    /**
     * Returns the user ID.
     * @return the user ID.
     */
    public String getUserId();

    /**
     * Returns the password.
     * Note that this method returns a reference to the password.
     * It is the caller's responsibility to zero out the password information
     * after it is no longer needed.
     * <p>
     * @return the password.
     */
    public char[] getPassword();

    /**
     * Stores an attribute in this credentials instance.
     * @param name a <code>String</code> specifying the name of the attribute
     * @param value the <code>Object</code> to be stored
     */
    public void setAttribute(String name, Object value);

    /**
     * Removes an attribute from this credentials instance.
     * @param name a <code>String</code> specifying the name of the attribute
     *             to remove
     */
    public void removeAttribute(String name);

    /**
     * Returns the value of the named attribute as an <code>Object</code>,
     * or <code>null</code> if no attribute of the given name exists.
     * @param name a <code>String</code> specifying the name of
     *             the attribute
     *
     * @return	an <code>Object</code> containing the value
     *          of the attribute, or <code>null</code> if
     *          the attribute does not exist
     */
    public Object getAttribute(String name);

    /**
     * Returns the names of the attributes available to this
     * credentials instance. This method returns an empty array
     * if the credentials instance has no attributes available to it.
     * @return a string array containing the names of the stored attributes
     */
    public String[] getAttributeNames();
}