/*
 * SimpleValue.java
 *
 * This file is part of the specification for the
 * Java Content Repository API
 */
package javax.jcr;

import javax.jcr.util.ISO8601;
import java.util.Calendar;

/**
 * A <code>DateValue</code> provides an implementation
 * of the <code>Value</code> interface representing a date value.
 *
 * @author Stefan Guggisberg
 */
public class DateValue extends BaseValue {

    public static final int TYPE = PropertyType.DATE;

    private Calendar date = null;

    /**
     * Constructs a <code>DateValue</code> object representing a date.
     *
     * @param date the date this <code>DateValue</code> should represent
     s     */
    public DateValue(Calendar date) {
        super(TYPE);
        this.date = date;
    }

    /**
     * Returns a new <code>DateValue</code> initialized to the value
     * represented by the specified <code>String</code>.
     * <p>
     * The specified <code>String</code> must be a ISO8601-compliant date/time
     * string.
     * @see javax.jcr.util.ISO8601
     * @param s the string to be parsed.
     * @return a newly constructed <code>DateValue</code> representing the
     *         the specified value.
     * @throws ValueFormatException If the <code>String</code> is not a valid
     *         ISO8601-compliant date/time string.
     */
    public static DateValue valueOf(String s) throws ValueFormatException {
        Calendar cal = ISO8601.parse(s);
        if (cal != null) {
            return new DateValue(cal);
        } else {
            throw new ValueFormatException("not a valid date format");
        }
    }

    //----------------------------------------------------------------< Value >
    /**
     * @see Value#getDate
     */
    public Calendar getDate() throws ValueFormatException, IllegalStateException, RepositoryException {
        setValueConsumed();

        if (date != null) {
            return date;
        } else {
            throw new ValueFormatException("empty value");
        }
    }

    /**
     * @see Value#getLong
     */
    public long getLong() throws ValueFormatException, IllegalStateException, RepositoryException {
        setValueConsumed();

        if (date != null) {
            return date.getTime().getTime();
        } else {
            throw new ValueFormatException("empty value");
        }
    }

    /**
     * @see Value#getBoolean
     */
    public boolean getBoolean() throws ValueFormatException, IllegalStateException, RepositoryException {
        setValueConsumed();

        if (date != null) {
            throw new ValueFormatException("cannot convert date to boolean");
        } else {
            throw new ValueFormatException("empty value");
        }
    }

    /**
     * @see Value#getDouble
     */
    public double getDouble() throws ValueFormatException, IllegalStateException, RepositoryException {
        setValueConsumed();

        if (date != null) {
            long ms = date.getTime().getTime();
            if (ms <= Double.MAX_VALUE) {
                return ms;
            }
            throw new ValueFormatException("conversion from date to double failed: inconvertible types");
        } else {
            throw new ValueFormatException("empty value");
        }
    }

    /**
     * @see Value#getString
     */
    public String getString() throws ValueFormatException, IllegalStateException, RepositoryException {
        setValueConsumed();

        if (date != null) {
            return ISO8601.format(date);
        } else {
            throw new ValueFormatException("empty value");
        }
    }
    
  public boolean equals(Object obj)
  {
    boolean equals = false;
    if (obj instanceof DateValue)
    {
      DateValue dateValue = (DateValue)obj;
      if (dateValue.date != null && date != null)
      {
        equals = dateValue.date.equals(date);
      }
      else if (dateValue.date == null && date == null)
      {
        equals = true;
      }
    }
    return equals;
  }
  
  public int hashCode()
  {
    int hashCode = 37;
    if (date != null)
    {
      hashCode = date.hashCode();
    }
    return hashCode;
  }
}
