/*
 * SimpleValue.java
 *
 * This file is part of the specification for the
 * Java Content Repository API
 */
package javax.jcr;

import java.util.Calendar;
import java.util.Date;

/**
 * A <code>LongValue</code> provides an implementation
 * of the <code>Value</code> interface representing a long value.
 *
 * @author Stefan Guggisberg
 */
public class LongValue extends BaseValue {

    public static final int TYPE = PropertyType.LONG;

    private Long lNumber = null;

    /**
     * Constructs a <code>LongValue</code> object representing a long.
     *
     * @param lNumber the long this <code>LongValue</code> should represent
     */
    public LongValue(Long lNumber) {
        super(TYPE);
        this.lNumber = lNumber;
    }

    /**
     * Constructs a <code>LongValue</code> object representing a long.
     *
     * @param l the long this <code>LongValue</code> should represent
     */
    public LongValue(long l) {
        super(TYPE);
        this.lNumber = new Long(l);
    }

    /**
     * Returns a new <code>LongValue</code> initialized to the value
     * represented by the specified <code>String</code>.
     * @param s the string to be parsed.
     * @return a newly constructed <code>LongValue</code> representing the
     *         the specified value.
     * @throws ValueFormatException If the <code>String</code> does not
     *         contain a parsable <code>long</code>.
     */
    public static LongValue valueOf(String s) throws ValueFormatException {
        try {
            return new LongValue(Long.parseLong(s));
        } catch (NumberFormatException e) {
            throw new ValueFormatException("invalid format", e);
        }
    }

    //----------------------------------------------------------------< Value >
    /**
     * @see Value#getDate
     */
    public Calendar getDate() throws ValueFormatException, IllegalStateException, RepositoryException {
        setValueConsumed();

        if (lNumber != null) {
            // loosing timezone information...
            Calendar cal = Calendar.getInstance();
            cal.setTime(new Date(lNumber.longValue()));
            return cal;
        } else {
            throw new ValueFormatException("empty value");
        }
    }

    /**
     * @see Value#getLong
     */
    public long getLong() throws ValueFormatException, IllegalStateException, RepositoryException {
        setValueConsumed();

        if (lNumber != null) {
            return lNumber.longValue();
        } else {
            throw new ValueFormatException("empty value");
        }
    }

    /**
     * @see Value#getBoolean
     */
    public boolean getBoolean() throws ValueFormatException, IllegalStateException, RepositoryException {
        setValueConsumed();

        throw new ValueFormatException("conversion to boolean failed: inconvertible types");
    }

    /**
     * @see Value#getDouble
     */
    public double getDouble() throws ValueFormatException, IllegalStateException, RepositoryException {
        setValueConsumed();

        if (lNumber != null) {
            return lNumber.doubleValue();
        } else {
            throw new ValueFormatException("empty value");
        }
    }

    /**
     * @see Value#getString
     */
    public String getString() throws ValueFormatException, IllegalStateException, RepositoryException {
        setValueConsumed();

        if (lNumber != null) {
            return lNumber.toString();
        } else {
            throw new ValueFormatException("empty value");
        }
    }
    
  public boolean equals(Object obj)
  {
    boolean equals = false;
    if (obj instanceof LongValue)
    {
      LongValue longValue = (LongValue)obj;
      if (longValue.lNumber != null && lNumber != null)
      {
        equals = longValue.lNumber.equals(lNumber);
      }
      else if (longValue.lNumber == null && lNumber == null)
      {
        equals = true;
      }
    }
    return equals;
  }
  
  public int hashCode()
  {
    int hashCode = 37;
    if (lNumber != null)
    {
      hashCode = lNumber.hashCode();
    }
    return hashCode;
  }
}
