/*
 * Property.java
 *
 * This file is part of the specification for the
 * Java Content Repository API
 */
package javax.jcr;

import javax.jcr.nodetype.PropertyDef;
import java.io.InputStream;
import java.util.Calendar;

/**
 * A <code>Property</code> object represents the smallest granularity of content
 * storage.
 * <p>
 * <b>Level 1 and 2</b>
 * <p>
 * A property must have one and only one parent node. A property does
 * not have children. When we say that node A "has" property B it means that B
 * is a child of A.
 * <p>
 * A property consists of a name and a value. See <code>{@link Value}</code>.
 *
 * @author Peeter Piegaze
 * @author Stefan Guggisberg
 */
public interface Property extends Item {

    /**
     * Sets the value of this <code>Property</code> to <code>value</code>.
     * If this property's property type is not constrained by the node type of
     * its parent node, then the property type is changed to that of the supplied
     * <code>value</code>. In
     * those implementations that support it (for example, client-server), a
     * <code>StaleValueException</code> is thrown if the property's value has changed
     * via another ticket since this <code>Property</code> instance was retrieved.
     * <p>
     * To persist the change <code>{@link Node#save()}</code> must be called on
     * the parent node of the property being set or a higher-order
     * ancestor of the property.
     *
     * @param value The new value to set the property to.
     *
     * @throws ValueFormatException if the type or format of the specified value
     * is incompatible with the <code>type</code> of this property.
     *
     * @throws StaleValueException if (in implementations where this is possible)
     * the value has changed via another Ticket, since this property was retrieved.
     *
     * @throws RepositoryException if another error occurs.
     */
    public void setValue(Value value) throws ValueFormatException, StaleValueException, RepositoryException;
    
    /**
     * Sets the values of this <code>Property</code> to <code>values</code>.
     * If this property's property type is not constrained by the node type of
     * its parent node, then the property type is changed to that of the supplied
     * <code>value</code>. In
     * those implementations that support it (for example, client-server), a
     * <code>StaleValueException</code> is thrown if the property's value has changed
     * via another ticket since this <code>Property</code> instance was retrieved.
     * <p>
     * To persist the change <code>{@link Node#save()}</code> must be called on
     * the parent node of the property being set or a higher-order
     * ancestor of the property.
     *
     * @param values The new values to set the property to.
     *
     * @throws ValueFormatException if the type or format of the specified values
     * is incompatible with the <code>type</code> of this property.
     *
     * @throws StaleValueException if (in implementations where this is possible)
     * the value has changed via another Ticket, since this property was retrieved.
     *
     * @throws RepositoryException if another error occurs.
     */
    public void setValue(Value[] values) throws ValueFormatException, StaleValueException, RepositoryException;    

    /**
     * Sets the value of this <code>Property</code> to <code>value</code>.
     * Same as <code>{@link #setValue(Value value)}</code> except that the
     * value is specified as a <code>String</code>.
     *
     * @param value The new value to set the property to.
     *
     * @throws ValueFormatException if the type or format of the specified value
     * is incompatible with the <code>type</code> of this property.
     *
     * @throws StaleValueException if (in implementations where this is possible)
     * the value has changed via another Ticket, since this property was retrieved.
     *
     * @throws RepositoryException if another error occurs.
     */
    public void setValue(String value) throws ValueFormatException, StaleValueException, RepositoryException;

    /**
     * Sets the value of this <code>Property</code> to <code>value</code>.
     * Same as <code>{@link #setValue(Value value)}</code> except that the
     * value is specified as a <code>boolean</code>.
     *
     * @param value The new value to set the property to.
     *
     * @throws ValueFormatException if the type or format of the specified value
     * is incompatible with the <code>type</code> of this property.
     *
     * @throws StaleValueException if (in implementations where this is possible)
     * the value has changed via another Ticket, since this property was retrieved.
     *
     * @throws RepositoryException if another error occurs.
     */
    public void setValue(boolean value) throws ValueFormatException, StaleValueException, RepositoryException;

    /**
     * Sets the value of this <code>Property</code> to <code>value</code>.
     * Same as <code>{@link #setValue(Value value)}</code> except that the
     * value is specified as a <code>long</code>.
     *
     * @param value The new value to set the property to.
     *
     * @throws ValueFormatException if the type or format of the specified value
     * is incompatible with the <code>type</code> of this property.
     *
     * @throws StaleValueException if (in implementations where this is possible)
     * the value has changed via another Ticket, since this property was retrieved.
     *
     * @throws RepositoryException if another error occurs.
     */
    public void setValue(long value) throws ValueFormatException, StaleValueException, RepositoryException;

    /**
     * Sets the value of this <code>Property</code> to <code>value</code>.
     * Same as <code>{@link #setValue(Value value)}</code> except that the
     * value is specified as a <code>double</code>.
     *
     * @param value The new value to set the property to.
     *
     * @throws ValueFormatException if the type or format of the specified value
     * is incompatible with the <code>type</code> of this property.
     *
     * @throws StaleValueException if (in implementations where this is possible)
     * the value has changed via another Ticket, since this property was retrieved.
     *
     * @throws RepositoryException if another error occurs.
     */
    public void setValue(double value) throws ValueFormatException, StaleValueException, RepositoryException;

    /**
     * Sets the value of this <code>Property</code> to <code>value</code>.
     * Same as <code>{@link #setValue(Value value)}</code> except that the
     * value is specified as a <code>InputStream</code>.
     *
     * @param value The new value to set the property to.
     *
     * @throws ValueFormatException if the type or format of the specified value
     * is incompatible with the <code>type</code> of this property.
     *
     * @throws StaleValueException if (in implementations where this is possible)
     * the value has changed via another Ticket, since this property was retrieved.
     *
     * @throws RepositoryException if another error occurs.
     */
    public void setValue(InputStream value) throws ValueFormatException, StaleValueException, RepositoryException;

    /**
     * Sets the value of this <code>Property</code> to <code>value</code>.
     * Same as <code>{@link #setValue(Value value)}</code> except that the
     * value is specified as a <code>Calendar</code>.
     *
     * @param value The new value to set the property to.
     *
     * @throws ValueFormatException if the type or format of the specified value
     * is incompatible with the <code>type</code> of this property.
     *
     * @throws StaleValueException if (in implementations where this is possible)
     * the value has changed via another Ticket, since this property was retrieved.
     *
     * @throws RepositoryException if another error occurs.
     */
    public void setValue(Calendar value) throws ValueFormatException, StaleValueException, RepositoryException;

    /**
     * Gets the value of this property.
     * Returns the value of the <code>Property</code> as a generic
     * <code>Value</code> object or <code>null</code> if the
     * <code>Property</code> has no value.
     *
     * @throws RepositoryException if an error occurs.
     *
     * @return the value
     */
    public Value getValue() throws RepositoryException;
    
    /**
     * Gets the values of this property.
     * Returns the values of the <code>Property</code> as a generic
     * <code>Value</code>[] object or <code>null</code> if the
     * <code>Property</code> has no value.
     *
     * @throws RepositoryException if an error occurs.
     *
     * @return the values
     */
    public Value[] getValues() throws RepositoryException;

    /**
     * Returns a <code>String</code> representation of the value of this <code>Property</code>.
     * If conversion is not possible, the property is empty, or an error occurs,
     * an empty string is returned. This behaviour is in contrast to
     * <code>{@link Value#getString()}</code> which
     * throws an exception on conversion or other error or if the property is
     * empty.
     *
     * @return A string representation of the value of this <code>Property</code>.
     */
    public String getString();

    /**
     * Returns a <code>double</code> representation of the value of this <code>Property</code>.
     * If conversion is not possible, the property is empty, or an error occurs,
     * 0.0 is returned. This behaviour is in contrast to
     * <code>{@link Value#getDouble()}</code> which
     * throws an exception on conversion or other error or if the property is
     * empty.
     *
     * @return A double representation of the value of this <code>Property</code>.
     */
    public double getDouble();

    /**
     * Returns an <code>InputStream</code> representation of the value of this
     * <code>Property</code>.
     * If conversion is not possible, the property is
     * empty, or an error occurs, an empty stream is returned. This behaviour
     * is in contrast to <code>{@link Value#getStream()}</code> which
     * throws an exception on conversion or other error or if the property is
     * empty.
     *
     * @return A stream representation of the value of this <code>Property</code>.
     */
    public InputStream getStream();

    /**
     * Returns a <code>Calendar</code> representation of the value of this <code>Property</code>.
     * If conversion is not possible, the property is empty, or an error occurs,
     * a <code>Calendar</code> of 1970-01-01T00:00:00Z is returned. This behaviour is in contrast to
     * <code>{@link Value#getDate()}</code> which
     * throws an exception on conversion or other error or if the property is
     * empty.
     *
     * @return A string representation of the value of this <code>Property</code>.
     */
    public Calendar getDate();

    /**
     * Returns a <code>boolean</code> representation of the value of this <code>Property</code>.
     * If conversion is not possible, the property is empty, or an error occurs,
     * a value of <code>false</code> is returned. This behavior is in contrast to
     * <code>{@link Value#getBoolean()}</code> which
     * throws an exception on conversion or other error or if the property is
     * empty.
     *
     * @return A string representation of the value of this <code>Property</code>.
     */
    public boolean getBoolean();

    /**
     * Returns a <code>long</code> representation of the value of this <code>Property</code>.
     * If conversion is not possible, the property is empty, or an error occurs,
     * <code>0</code> is returned. This behaviour is in contrast to
     * <code>{@link Value#getLong()}</code> which
     * throws an exception on conversion or other error or if the property is
     * empty.
     *
     * @return A string representation of the value of this <code>Property</code>.
     */
    public long getLong();

    /**
     * Indicates whether this property has a value.
     * <p>
     * <b>Level 1 and 2:</b>
     * <p>
     * Returns true if this property has a value. A property with no value
     * is the same as a property with value set to <code>null</code>.
     *
     * @return <code>true</code> if the property has content;
     *         <code>false</code> otherwise.
     */
    public boolean hasValue();

    /**
     * Returns the length of the value of this property.
     * <p>
     * <b>Level 1 and 2:</b>
     * <p>
     * Returns the length of the value of this property in bytes if the value
     * is a <code>PropertyType.BINARY</code>, otherwise it returns the number
     * of characters needed to display the value (for strings this is the string
     * length, for numeric types it is the number of characters needed to
     * display the number). Returns 1 if the implementation cannot determine
     * the length. Returns 0 if the property has no value.
     *
     * @return length
     */
    public long getLength();
        
    /**
     * Returns the type of the property
     * <p>
     * <b>Level 1 and 2:</b>
     * <p>
     * Returns the type of this property.  Merely acts as a passthrough to 
     * <code>Value.getType()</code>.
     *
     * @return type
     */
    public int getType();

    /**
     * Returns the definition of <i>this</i> <code>Property</code>. This method
     * is actually a shortcut to searching through this property's parent's
     * node type (and its supertypes) for the property definition applicable
     * to this property.
     *
     * @see javax.jcr.nodetype.NodeType#getPropertyDefs
     *
     * @return a <code>PropertyDef</code> object.
     */
    public PropertyDef getDefinition();
}
