/*
 * SimpleValue.java
 *
 * This file is part of the specification for the
 * Java Content Repository API
 */
package javax.jcr;

import java.util.Calendar;

/**
 * A <code>SoftLinkValue</code> provides an implementation
 * of the <code>Value</code> interface representing a soft link value
 * (i.e. an absolute repository path).
 *
 * @author Stefan Guggisberg
 */
public class SoftLinkValue extends BaseValue {

    public static final int TYPE = PropertyType.SOFTLINK;

    private String absPath = null;

    /**
     * Constructs a <code>SoftLinkValue</code> object representing an absolute
     * path.
     *
     * @param absPath the link this <code>SoftLinkValue</code> should represent
     */
    public SoftLinkValue(String absPath) {
        super(TYPE);
        this.absPath = absPath;
    }

    //----------------------------------------------------------------< Value >
    /**
     * @see Value#getDate
     */
    public Calendar getDate() throws ValueFormatException, IllegalStateException, RepositoryException {
        setValueConsumed();

        throw new ValueFormatException("conversion to date failed: inconvertible types");
    }

    /**
     * @see Value#getLong
     */
    public long getLong() throws ValueFormatException, IllegalStateException, RepositoryException {
        setValueConsumed();

        throw new ValueFormatException("conversion to long failed: inconvertible types");
    }

    /**
     * @see Value#getBoolean
     */
    public boolean getBoolean() throws ValueFormatException, IllegalStateException, RepositoryException {
        setValueConsumed();

        throw new ValueFormatException("conversion to boolean failed: inconvertible types");
    }

    /**
     * @see Value#getDouble
     */
    public double getDouble() throws ValueFormatException, IllegalStateException, RepositoryException {
        setValueConsumed();

        throw new ValueFormatException("conversion to double failed: inconvertible types");
    }

    /**
     * @see Value#getString
     */
    public String getString() throws ValueFormatException, IllegalStateException, RepositoryException {
        setValueConsumed();

        if (absPath != null) {
            return absPath;
        } else {
            throw new ValueFormatException("empty value");
        }
    }
    
  public boolean equals(Object obj)
  {
    boolean equals = false;
    if (obj instanceof SoftLinkValue)
    {
      SoftLinkValue softLinkValue = (SoftLinkValue)obj;
      if (softLinkValue.absPath != null && absPath != null)
      {
        equals = softLinkValue.absPath.equals(absPath);
      }
      else if (softLinkValue.absPath == null && absPath == null)
      {
        equals = true;
      }
    }
    return equals;
  }
  
  public int hashCode()
  {
    int hashCode = 37;
    if (absPath != null)
    {
      hashCode = absPath.hashCode();
    }
    return hashCode;
  }
}
