/*
 * SimpleValue.java
 *
 * This file is part of the specification for the
 * Java Content Repository API
 */
package javax.jcr;

import javax.jcr.util.ISO8601;
import java.util.Calendar;

/**
 * A <code>StringValue</code> provides an implementation
 * of the <code>Value</code> interface representing a string value.
 *
 * @author Stefan Guggisberg
 */
public class StringValue extends BaseValue {

    public static final int TYPE = PropertyType.STRING;

    private String text = null;

    /**
     * Constructs a <code>StringValue</code> object representing a string.
     *
     * @param text the string this <code>StringValue</code> should represent
     */
    public StringValue(String text) {
        super(TYPE);
        this.text = text;
    }

    //----------------------------------------------------------------< Value >
    /**
     * @see Value#getDate
     */
    public Calendar getDate() throws ValueFormatException, IllegalStateException, RepositoryException {
        setValueConsumed();

        if (text != null) {
            Calendar cal = ISO8601.parse(text);
            if (cal == null) {
                throw new ValueFormatException("not a valid date format");
            } else {
                return cal;
            }
        } else {
            throw new ValueFormatException("empty value");
        }
    }

    /**
     * @see Value#getLong
     */
    public long getLong() throws ValueFormatException, IllegalStateException, RepositoryException {
        setValueConsumed();

        if (text != null) {
            try {
                return Long.parseLong(text);
            } catch (NumberFormatException e) {
                throw new ValueFormatException("conversion to long failed", e);
            }
        } else {
            throw new ValueFormatException("empty value");
        }
    }

    /**
     * @see Value#getBoolean
     */
    public boolean getBoolean() throws ValueFormatException, IllegalStateException, RepositoryException {
        setValueConsumed();

        if (text != null) {
	    return Boolean.valueOf(text).booleanValue();
        } else {
            throw new ValueFormatException("empty value");
        }
    }

    /**
     * @see Value#getDouble
     */
    public double getDouble() throws ValueFormatException, IllegalStateException, RepositoryException {
        setValueConsumed();

        if (text != null) {
            try {
                return Double.parseDouble(text);
            } catch (NumberFormatException e) {
                throw new ValueFormatException("conversion to double failed", e);
            }
        } else {
            throw new ValueFormatException("empty value");
        }
    }

    /**
     * @see Value#getString
     */
    public String getString() throws ValueFormatException, IllegalStateException, RepositoryException {
        setValueConsumed();

        if (text != null) {
            return text;
        } else {
            throw new ValueFormatException("empty value");
        }
    }
    
  public boolean equals(Object obj)
  {
    boolean equals = false;
    if (obj instanceof StringValue)
    {
      StringValue stringValue = (StringValue)obj;
      if (stringValue.text != null && text != null)
      {
        equals = stringValue.text.equals(text);
      }
      else if (stringValue.text == null && text == null)
      {
        equals = true;
      }
    }
    return equals;
  }
  
  public int hashCode()
  {
    int hashCode = 37;
    if (text != null)
    {
      hashCode = text.hashCode();
    }
    return hashCode;
  }
}
