/*
 * Event.java
 *
 * This file is part of the specification for the
 * Java Content Repository API
 */
package javax.jcr.observation;

import javax.jcr.Item;
import javax.jcr.Ticket;

/**
 * All events used by the ObservationManager system are subclassed from this interface
 * <p>
 * <b>Level 2 only</b>
 * <p>
 * For details see the <i>ObservationManager</i> section of the JCR standard document.
 * @author tima, ppiegaze
 */
public interface Event {

    /**
     * Returns the type of this event. One of:
     * <ul>
     * <li>EventType.ITEM_ADDED
     * <li>EventType.ITEM_CHANGED
     * <li>EventType.ITEM_REMOVED
     * <li>EventType.ITEM_VERSIONED
     * <li>EventType.LABEL_ADDED
     * <li>EventType.LABEL_REMOVED
     * <li>EventType.ITEM_LOCKED
     * <li>EventType.ITEM_UNLOCKED
     * <li>EventType.LOCK_EXPIRED
     * </ul>
     *
     * @return the type of this event.
     */
    public long getType();

    /**
     * Returns the item connected with this event.
     *
     * @return an <code>Item</code>.
     */
    public Item getItem();

    /**
     * Returns the Ticket connected with this event
     *
     * @return an <code>Ticket</code>.
     */
    public Ticket getTicket();

    /**
     * Returns the ID of the user who triggered this event.
     * Equivalent to <code>Event.getTicket().getUserId()</code>
     *
     * @return the ID of the user who triggered this event.
     */
    public String getUserId();

    /**
     * Returns true if this event is a redelivery of a previously delivered
     * event. This may happen if the repository notified the listener, but the
     * operation failed due to a system error or the listener throws a runtime
     * exception. See <i>Quality of Service</i> in the <i>ObservationManager</i>
     * section of the JCR standard document.
     *
     * @return a <code>boolean</code>
     */
    public boolean getRedelivered();
}
